#include <string>
#include <map>

#include "vect.h"
#include "common.h"

namespace CO_common {
  
  Cmdl_parser::Cmdl_parser(int argc, char *argv[])
  {
    for (int i=0; i<argc; i++) {
      arg.push_back(argv[i]);
    }
  }
  void Cmdl_parser::add_opt(const string &h, const string &p, string *s)
  {
    Cmdl_data d(h, CLT_STR);
    d.data.s = s;
    opt.insert(Opt_map::value_type(p, d));
  }
  void Cmdl_parser::add_opt(const string &h, const string &p,  int *i)
  {
    Cmdl_data d(h, CLT_INT);
    d.data.i = i;
    opt.insert(Opt_map::value_type(p, d));
  }
  void Cmdl_parser::add_opt(const string &h, const string &p, float *v)
  {
    Cmdl_data d(h, CLT_FLOAT);
    d.data.f = v;
    opt.insert(Opt_map::value_type(p, d));
  }
  void Cmdl_parser::add_opt(const string &h, const string &p, bool *b)
  {
    Cmdl_data d(h, CLT_BOOL);
    d.data.b = b;
    opt.insert(Opt_map::value_type(p, d));
  }
  /** @name process cmdl
   * main interface of this module. It does the processing of
   * command line options.
   */
  bool Cmdl_parser::process_cmdl()
  {
    Vector<string>::iterator it;
    for (it=arg.begin(), it++; it<arg.end(); it++)
      if (!process_this_arg(it))
	return false;
    // now validate values just parsed
    return true;
  }
  void Cmdl_parser::remove_arg(int i)
  {
    Vector<string>::iterator it = arg.begin();
    arg.erase(it+i);
  }
  bool Cmdl_parser::is_num(string &s) const
  {
    const char *c = s.c_str();
    while (*c){
      if (!isdigit(*c)) {
	return false;
      }
      c++;
    }
    return true;
  }
  /** @name print option
   * used to print the options present in the opt structure */
  void Cmdl_parser::print_options()
  {
    Opt_map::const_iterator it;
    for (it = opt.begin(); it != opt.end(); it++)
      cout << "    " << it->first  << "  \t-- "
	   <<  it->second.help <<endl;
  }

  bool Cmdl_parser::process_this_arg(Vector<string>::iterator &it)
  {
    Opt_map::iterator mi = opt.find(*it);
    if (mi == opt.end()) {
      cout << "Unknow option " << *it << endl;
      return false;
    }
    switch (mi->second.type){
    case CLT_BOOL:
      *(mi->second.data.b) = true;
      return true;
    
    case CLT_FLOAT:
      *(mi->second.data.f) = atof(it->c_str());
      return true;
    case CLT_INT:
      it++;
      if (it == arg.end()){  // no more input
        cerr << "Error: No argument for parameter " << *it <<endl;
        return false;
      }
      if (!is_num(*it)) {
        cerr << "Argument " << *it << "to parameter "
             << *(it-1) << " is not an int." << endl;
        return false;
      }
      *(mi->second.data.i) = atoi(it->c_str());
      return true;

    case CLT_STR:
      it++;
      if (it == arg.end()){  // no more input
        cerr << "Error: No argument for parameter " << *it <<endl;
        return false;
      }
      *(mi->second.data.s) = *(it);
      return true;

    default:
      cerr << "Error: invalid type for command line option\n";
    }
    return false;
  }
}
