#if !defined(_FAP_FGRASP_H)
#define _FAP_FGRASP_H

#include "grasp.h"
#include "sol.h"

bool verbose();

class Bc {
 public:
  typedef set< int, less < int > > Freq_set;
  Bc () : bc() { }
  bool operator< (Bc &c) {
    // this is unusual but easier to do ...
    return bc.size() > c.bc.size();
  }
  int num_blocked()   { return bc.size(); }
  bool blocked(int i) { return bc.find(i) != bc.end(); }
  void block(int i)   { bc.insert(i); }
  void clear() { bc.clear(); }
 private:
  Freq_set bc;
};

struct Ord_Bc {
  Bc *p;
  bool operator<(const Ord_Bc &b) const { return (*p) < (*(b.p)); }
  Ord_Bc(Bc &b) : p(&b) {}
  Ord_Bc(const Ord_Bc &b) : p(b.p) {}
};


class FGrasp : public Grasp<FAP_inst,FAP_sol> {
public:
  FGrasp(FAP_inst &i, int max_iter_ = 1000);
  FGrasp(FAP_inst &i, FAP_sol &s, int max_iter_ = 1000);
  virtual ~FGrasp();
  virtual void constructor(int alpha);
  virtual void local_search();
  virtual void update();
  void init_constructor();
  bool stop_condition() {  
    return Grasp<FAP_inst,FAP_sol>::stop_condition() 
       || iter_num() > max_iter 
       || found_look4();    
  }
 private:
  // maitains the order or elements in Bc
  Vector<Ord_Bc> ord_bc;
  int max_iter;
  int remaining_trans;

  void update_bc(int trans, int freq);
  int choose_channel(int trans);
  int choose_trans(int trans);
protected:
 // vector of blocked channels
  Vector<Bc> bc;
};

#endif /* _FAP_GRASP_H */
