#if !defined(_FAP_GRASP_H)
#define _FAP_GRASP_H

#include <set>
#include "solver.h"
#include "vect.h"

template <class T,class Sol>
class Grasp : public solver<T,Sol> {
protected:
  Sol *best;
private:
  int iter;
  bool delete_best;
  int look4;
public:
  Grasp(T &i) :
    solver<T,Sol>(i), best(new Sol(i)), iter(0), 
    delete_best(true), look4(-1) {}
  Grasp(T&i, Sol &s) :
    solver<T,Sol>(i), best(&s), iter(0), 
    delete_best(false), look4(-1) {}
  virtual ~Grasp() { if (delete_best) delete best;}
  virtual Sol *solve();
  virtual void init() {}
  virtual int  get_alpha() { return ALPHA; }
  virtual void constructor(int alpha) = 0;
  virtual void local_search() = 0;
  virtual void update();
  virtual void terminate() {}
  /** This function returns the number of the current iteration.
       Iteration numbers are incremented after function `update'
       is called. */
  int iter_num();
  void set_look4(int value) { look4 = value; }
  bool found_look4() { 
    if (look4 >= 0) return (best->get_of() <= look4); 
    else            return false;
  }
};

#endif /* _FAP_GRASP_H */
