#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string.h>
#include <ctype.h>

#include "iformat.h"
#include "inst.h"

bool _Resize(void *& p, size_t size, size_t old)
{
  void *pp = p;
  p = new char[size];
  if (!p) {
    p = pp;
    return false;
  }
  size_t i;
  char *pc, *ppc;
  pc = (char *)p;
  ppc = (char *)pp;
  for (i=0; i<old && i<size; i++){
    pc[i] = ppc[i];
  }
  return true;
}

#define Resize(p, type, size, old) \
  _Resize((void *)p, sizeof(type)*(size), sizeof(type)*(old))


void eat_white_space(ifstream &is)
{
  char c;
  do {
    is.get(c);
    
    /* eats comments */
    if ( c == '#' ) {
      do {
	is.get(c);
      } while ( c != '\n' && c != '\r' );
    }
  } while (isspace(c));
  is.putback(c);
}

void eat_text(ifstream &is)
{
  char c, delimiter;
  eat_white_space(is);
  is.get(delimiter);
  do {
    is.get(c);
  } while (c != delimiter);
}

void print_next_token(ifstream &is)
{
  char c;
  eat_white_space(is);
  is.get(c);
  while (isalnum(c) || c == '-' || c == '_' ) {
    is.get(c);
  }
  is.putback(c);
}

static const int INITIAL_BUFFER_SIZE = 32;
static const int MAX_NUMBER_SIZE = 32;

int read_float(float *num, ifstream &is)
{
  char c, cc;
  int i, npt = 0, exp = 0;
  char buf[MAX_NUMBER_SIZE];

  eat_white_space(is);
  i = 0;
  is.get(c);
  for (; i < MAX_NUMBER_SIZE
	 && (isdigit(c) 
             || (c == '.' && ++npt == 1 && exp == 0)
             || (c == 'e' && ++exp == 1)
             || (c == '-' && (i==0 || cc == 'e'))); i++) {
    buf[i] = (char)c;
    cc = c;
    is.get(c);
  }
  is.putback(c);
  if (i == 0) {
    return 0;
  }
  if (i == MAX_NUMBER_SIZE) {
    cerr << "error: integer is too large.\n";
    return 0;
  }
  buf[i] = 0;
  // Converts the number in buf to decimal
  //
  *num = atof(buf); 
  return 1;
}

int read_integer(int *num, ifstream &is)
{
  int i;
  char c, buf[MAX_NUMBER_SIZE];

  eat_white_space(is);
  i = 0;
  is.get(c);
  for (; isdigit(c) && i < MAX_NUMBER_SIZE; i++) {
    buf[i] = (char)c;
    is.get(c);
  }
  is.putback(c);
  if (i == 0) {
    return 0;
  }
  if (i == MAX_NUMBER_SIZE) {
    cerr <<"error: integer is too large.\n";
    return 0;
  }
  buf[i] = 0;
  // Converts the number in buf to decimal
  //
  *num = atoi(buf);
  return 1;
}

int read_word(char *w, int size, ifstream &is)
{
  int i;
  char c;
  eat_white_space(is);
  is.get(c);
  for (i=0; i<size && (isalnum(c) || c == '-' || c == '_' ); i++) {
    w[i] = (char)c;
    is.get(c);
  }
  w[i] = 0;
  is.putback(c);
  if (i == 0) {
    return 0;
  }
  if (i == size) {
    cerr <<"error: word is too large (> " << size << " chars)\n";
    return 0;
  }
  return 1;
}

int read_token(char *token, ifstream &is)
{
  int size, i=0;
  char c;
  static char *buf=0;
  static int buf_size = INITIAL_BUFFER_SIZE;

  if (!buf) {
    buf = new char[buf_size];
    if (!buf) goto e1;
  }
  size = strlen(token);
  if (size >= buf_size) {
    delete buf;
    buf = new char[buf_size*2];
    if (!buf) goto e1;
    buf_size *= 2;
  }
  eat_white_space(is);
  while (*token) {
    is.get(c);
    buf[i] = (char)c;
    i++;
    if (tolower(c) != tolower(*token)) {
      /* put token read back to stream */
      for (i--;i>=0; i--){
	is.putback(buf[i]);
      }
      return 0;
    }
    token++;
  }
  return 1;
 e1:
  cerr <<"error: not more memory available. Goodbye.\n";
  exit(1);
}

cell_desc *read_demand(int *n_cells, ifstream &is)
{
  char *to_match, ident[MAX_IDENT_SIZE];
  int c, i, cell_num, size = INITIAL_DEMAND_VEC_SIZE;
  cell_desc *d;
  float f;

  d = new cell_desc[size];
  if (!d) goto e1;

  m(CELLS); match("{"); 
  for (c = 0; read_integer(&cell_num, is); c++) {
    match("{");
    readw;m(;);     /* site identifier */
    readi(i);m(;);  /*sector number */ 
    readf(f);m(;);  /* demand */
    if (c >= size) {
      if (!Resize(d, cell_desc, size*2, size)) {
	cerr << "memory exausted in read_demand. Goodbye.\n";
	delete d;
	return 0;
      }
      size *= 2;
    }
    strcpy(d[c].site, ident);
    d[c].cell = cell_num;
    /* document says demand is a float, but this never happens... */
    d[c].demand = (int) f;
    if (read_token("LOC", is)) {
      match("(");readf(f);match(",");readf(f);match(")");m(;);
    }
    if (read_token("LBC", is)) {
      while (read_integer(&i, is)); m(;);
    }
    if (read_token("CCS", is)) {
      readi(i);m(;);
    }

    match("}");
  }
  
  match("}");
  *n_cells  = c;
  return d;

 e1:
  return 0;
 error_ident:
  cerr << "error reading identifier.\n";
 error_float:
  cerr << "error reading float value.\n";
  return 0;
 error_integer:
  cerr << "error reading integer value.\n";
  return 0;
 error_match:
  cerr << "string " << to_match << " not found\n";
  return 0;
}

bool FAP_inst::read_iformat(ifstream &is)
{ 
  char *to_match, ident[MAX_IDENT_SIZE];
  cell_desc *demand;
  int cell1, cell2, min, max, cosep, cocell_sep, dummy_i;
  int i, j, n_trans, n_cells;
  float dummy_f;

  m(FORMAT); m({);  m(TYPE); m(SCENARIO); m(;);m(VERSION);
  if (!read_float(&dummy_f, is) || dummy_f != 1.0) {
    cerr << "wrong version of format! version is "<< dummy_f << "\n";
    exit(1);
  }
  m(;); match("}");
  m(GENERAL_INFORMATION); match("{");
  m(SCENARIO_ID);
  readw; m(;);
  if (read_token("ANNOTATION", is)) {
    eat_text(is);
    m(;);
  }
  m(NETWORK_TYPE);
  readw; m(;);
  m(SPECTRUM);
  match("(");
  readi(min);
  match(",");
  readi(max);
  if (min < 1 || min >= max) {
    cerr << "error: spectrum must be positive\n";
    exit(1);
  }
  spectrum_ini = min;
  n_freqs = max - min + 1;
  match(")"); m(;);
  if (read_token("GLOBALLY_BLOCKED_CHANNELS", is)) {
    while (read_integer(&dummy_i, is)); m(;);
  }
  m(CO_SITE_SEPARATION);
  readi(cosep);m(;);
  m(DEFAULT_CO_CELL_SEPARATION);
  readi(cocell_sep);m(;);
  m(HANDOVER_SEPARATION);
  for (i=0; i<4; i++) {
    readi(dummy_i);
  }
  match(";");
  if (read_token("MINIMAL_SIGNIFICANT_INTERFERENCE", is)){
    readf(dummy_f);m(;);
  }
  if (read_token("MAXIMAL_TOLERABLE_INTERFERENCE", is)) {
    readf(dummy_f);m(;);
  }
  m(DEMAND_MODEL);
  if (!read_token("ABSOLUTE", is)) {
    m(NORMALIZED_TRAFFIC);
  }
  m(;)m(SITE_LOCATIONS);
  readi(dummy_i);
  if (dummy_i != 1 && dummy_i != 0) {
    cerr << "error reading boolean value. value is "<< dummy_i << "\n";
    exit(1);
  }
  m(;)match("}");

  demand = read_demand(&n_cells, is);
  if (!demand) {
    cerr << "error reading demand.\n";
    exit(1);
  }

  n_trans = 0;
  for (i = 0; i<n_cells; i++)
    n_trans += demand[i].demand;
  
  init_vectors(n_cells, n_trans);
  
  int sum;
  for (i=j=1; i<=n_cells; i++) {
    set_demand(i, demand[i-1].demand);
      
    sum = j+demand[i-1].demand;
    for (; j<sum; j++)
      set_trans_bs(j, i);
  }
  
  m(CELL_RELATIONS)match("{");
  for (i=1; read_integer(&cell1, is); i++) {
    readi(cell2); match("{");
    for (j=0; j<n_cells; j++) {
      if (cell1 == demand[j].cell) {
        cell1 = j;
        break;
      }
    }
    if (j == n_cells) { cerr << "error reading relation\n"; exit(1); }
    for (j=0; j<n_cells; j++) {
      if (cell2 == demand[j].cell) {
        cell2 = j;
        break;
      }
    }  
    if (j == n_cells) { cerr << "error reading relation\n"; exit(1); }

    /* define correct co-site interference */
    if (!strcmp(demand[cell1].site,demand[cell2].site)) {
      set_dist(cell_num(cell1), cell_num(cell2), cosep);
    }

    if (read_token("H", is)) {   /* handover (yes or no) */
      readi(dummy_i);
      if (dummy_i != 1 && dummy_i != 0) {
	cerr << "error reading boolean value.\n";
	exit(1);
      }
      m(;);
    }
    if (read_token("S", is)) {   /* separation */
      readi(dummy_i);m(;);
      set_dist(cell_num(cell1), cell_num(cell2), dummy_i);
    }
    if (read_token("DA", is)) {   /* interference */
      readf(dummy_f);
      set_co_int(cell_num(cell1),cell_num(cell2), dummy_f);
      if (!read_float(&dummy_f, is)) {
	dummy_f = 0;
      }
      set_ad_int(cell_num(cell1),cell_num(cell2), dummy_f);
      m(;);
    }
    
    if (read_token("UA", is)) {   /* interference */
      readf(dummy_f);
      if (!read_float(&dummy_f, is)) {
	dummy_f = 0;
      }
      m(;);
    }
    
    if (read_token("DT", is)) {   /* interference */
      readf(dummy_f);
      if (!read_float(&dummy_f, is)) {
	dummy_f = 0;
      }
      m(;);
    }
    
    if (read_token("UT", is)) {   /* interference */
      readf(dummy_f);
      if (!read_float(&dummy_f, is)) {
	dummy_f = 0;
      }
      m(;);
    }
    match("}");
  }
  match("}");

  for (i=1; i<=n_cells; i++)
    // set minimum distance inside the cell
    set_dist(i, i, cocell_sep);

  free(demand);
  return true;
  
  error_ident:
  cerr << "error reading identifier.\n";
  return false;
  error_float:
  cerr << "error reading float value.\n";
  return false;
  error_integer:
  cerr << "error reading integer value.\n";
  return false;
  error_match:
  cerr << "string " << to_match << " not found\n";
  return false;
}

