#if !defined(_FAP_INST_H)
#define _FAP_INST_H

#include <map>
#include "vect.h"

//#define VEC

class ifstream;

class FAP_inst {
  typedef map<int, int, less<int> > mapi;
  typedef map<int, float, less<int> > mapf;
  int n_freqs;       // number of allowed frequencies
  int n_trans;       // numb. of antennas
  int n_bs;          // numb. of base stations
#ifdef VEC
  Vector<mapi> d;   // distance matrix
  Vector<mapf> co;  // co-site interference
  Vector<mapf> ad;  // adjcency interference
#else
  int **d;
  float **co, **ad;
#endif
  Vector<int> demand;   // demand at each transmitter
  Vector<int> trans_bs; // in which base station?
  int spectrum_ini;

 public:
  FAP_inst();
  ~FAP_inst();
  
  /// This function reads a new instance of the FAP from file `ifile'.
  /// Parameter iformat is true only if the instance is in the fap.zib.de
  /// format.
  /// @returns a pointer to the new instance
  static FAP_inst *create_instance(ifstream &ifile, bool iformat=false);
  
  void set_n_freqs(int i) { n_freqs = i;    }
  int num_bs()            { return n_bs;    }
  int num_trans()         { return n_trans; }
  int num_freq()          { return n_freqs; }
  int min_dist(int i, int j);
  float co_int(int i, int j);
  float ad_int(int i, int j);
  int cell(int i) { return trans_bs[i]; }
  int get_spectrum_ini() { return spectrum_ini; }
  void print();
 private:
  bool read_inst(ifstream &ifile, bool iformat=false);
  bool read_rand_instance(ifstream &ifile);
  bool read_iformat(ifstream &ifile);
  void set_dist(int i, int j, int dist);
  void set_co_int(int i, int j, float interf);
  void set_ad_int(int i, int j, float interf);
  void set_demand(int t, int d)   { demand[t] = d;   }
  void set_trans_bs(int t, int b) { trans_bs[t] = b; }
  void init_vectors(int nb, int nt);
  void b(int i, int j)
  {
    if (i < 1 || j < 1 || i > n_bs || j > n_bs) 
      (cerr << "error: acessing " << i << " " << j << endl).flush();
  }
  void exchange(int &i, int &j) {
    if (i > j) { int t = i; i = j; j = t; }
  }
  /// tests if this instance is symmetric: the value of the
  /// property p(x,y) is the same is p(y,x)
  bool symmetric();
};

#endif /* _FAP_INST_H */
