#include <algorithm>
#include "pathrel.h"
#include "lsearch.h"

const int FPathRel::ELITE_SET_SIZE = 5;
static const int MAX_PR_TRIES = 5;

/** Constructor */
FPathRel::FPathRel(FAP_inst & i, int max_iter):
FReactGrasp(i, max_iter) {}

FPathRel::FPathRel(FAP_inst & i, FAP_sol &s, int max_iter):
FReactGrasp(i, s, max_iter) {}

FPathRel::~FPathRel(){}

void create_rand_ordering(Vector<int> &a)
{
  fill(a.begin(), a.end(), 0);
  int n = a.size();
  for (int i=1; i<n; i++){
    int r = rand() % n;
    while (a[r]!=0)
      r = (r==(n-1) ? 0 : r+1);
    a[r] = i;
  }
}
/** called at the end of each iteration if GRASP constructor. */
void FPathRel::update()
{
  Vector<FAP_sol> &e = elite_set;
  if (e.size() >= (unsigned int)ELITE_SET_SIZE) {
    // insert solution only it its better then the best ...
    if (*sol < *(min_element(e.begin(), e.end())) ||
        // ... or it is better then the worst ...
        (*sol < *(max_element(e.begin(), e.end())) &&
          // and is not already in the set
          find(e.begin(), e.end(), *sol) == e.end()))
    {
      // remove the worst solution in the set
      e.erase(max_element(e.begin(), e.end()));
      // and insert the new.
      e.push_back(*sol);
    }
  } else { // just insert the current solution into elite set
    elite_set.push_back(*sol);
  }
  FReactGrasp::update();
}
/** Improves local search using path relinking */
void FPathRel::local_search()
{
  FReactGrasp::local_search();
  if (elite_set.size() >= (unsigned int)ELITE_SET_SIZE)
    path_relinking();
}
/** This function implements the path relinking step */
void FPathRel::path_relinking()
{
  if (verbose())(cout << "comecando path relinking\n").flush();
  // choose a solution in the elite_set
  const FAP_sol &s = get_elite_solution();
  do_path_relinking(s, *sol);
  if (verbose())(cout << "end of path relinking\n").flush();
}

const FAP_sol &FPathRel::get_elite_solution()
{
  int sol_number = (rand()%ELITE_SET_SIZE);
  return *(elite_set.begin()+sol_number);
}

void FPathRel::do_path_relinking(const FAP_sol &s, FAP_sol &s2)
{
  // try at most MAX_PR_TRIES improvements
  for (int k=0; k<MAX_PR_TRIES && !stop_condition(); k++) {
    // choose the best improvement
    FAP_sol new_sol = s2, best_sol = s2;
    int trans = 0;
    for (int i=1; i<=inst.num_trans() && !stop_condition(); i++){
      //if (rand()%3 == 0) continue;
      int oldf = new_sol.get_channel(i);
      new_sol.set_channel(i, s.get_channel(i));
      new_sol.calc_of();
      if (new_sol < best_sol) {
         if (trans != 0)
           best_sol.set_channel(trans, s2.get_channel(trans));
         best_sol.set_channel(i, new_sol.get_channel(i));
         best_sol.calc_of();
         trans = i;
      }
      new_sol.set_channel(i, oldf);
    }
    new_sol.calc_of();
    assert(new_sol == s2);
    if (best_sol < s2)
      s2 = best_sol;
  }
}


