#ifndef FAP_PATHREL_H
#define FAP_PATHREL_H

#include <set>
#include "rgrasp.h"

/** This class implements Path Relinking for the Frequency
    Assignment Problem.
    
  *@author Carlos A.S. Oliveira */
class FPathRel : public FReactGrasp {
public: 
  /** Constructor */
   FPathRel(FAP_inst &i, int max_iter);
   FPathRel(FAP_inst &i, FAP_sol &s, int max_iter);
	~FPathRel();
protected:	
  /** do path relinking from s tp s2.  s2 is changed, but
   * not s. */
  void do_path_relinking(const FAP_sol &s, FAP_sol &s2);
  /** This represents the elite set, which are the best
      solutions found in previous runs. */
  Vector<FAP_sol> elite_set;
  /** Size of the elite set */
  const static int ELITE_SET_SIZE;
  const FAP_sol &FPathRel::get_elite_solution();
  /** Improves local search using path relinking */
  virtual void local_search();
private: // Private attributes
  /** called at the end of each iteration if GRASP constructor. */
  virtual void update();
  /** This function implements the path relinking step */
  void path_relinking();
};

#endif
