#if !defined(_FAP_SOL_H)
#define _FAP_SOL_H

#include "inst.h"

#define UNF_WEIGHT 100000

/// This class represents a solution to the FAP.
class FAP_sol {
  FAP_inst *fi;      // instance of the problem
  long int unf;      // unfesibility
  double interf;     // interference
  Vector<int> freq;  // set of frequencies assigned to verts

  double interference();
  double unfeasibility();
  /// calculates and return the interference due only to one transmitter
  double FAP_sol::interference_due(int trans);
  /// calculates and return the unfeasibility due only to one transmitter
  double FAP_sol::unfeasibility_due(int trans);

public:
  // =============
  /// constructors
  FAP_sol() : fi(0), unf(0), interf(0) {}
  /// constructor
  // @parameter i an instance of the FAP
  FAP_sol(FAP_inst &i);
  /// Copy constructor
  // @parameter s another solution
  FAP_sol(const FAP_sol &s);

  /// creates a random solution
  void rand_sol();
  
  /// comparison operator
  bool operator<(const FAP_sol &) const;

  /// assignment operator
  FAP_sol &operator= (const FAP_sol &);

  /// returns the channel assigned to transmitter `trans'
  int get_channel(int trans) const { return freq[trans]; }

  /// changes the channel assigned to transmitter `trans' to `f'
  void set_channel(int trans, int f) { freq[trans] =  f;  }

  /// computes the objective value of this solution.
  void calc_of()
  { assert(fi != 0); interference(); unfeasibility(); }

  /// changes frequency of `trans' to `f' only if this improves
  /// objective function.
  /// @return true if there is an improvement, false otherwise.
  bool change_freq_if_improves(int trans, int f);

  /// returns an numerical representation of the objective value
  double get_of() const { return unf*(UNF_WEIGHT) + interf; }
  
  /// Equal operator
  bool operator==(const FAP_sol &s) const;

  /// read the solution from an input stream
  void input_sol(ifstream &);
  
  /// print the solution to an output stream. */
  friend ostream &operator<<(ostream &o, FAP_sol &s);
};

#endif /* _FAP_SOL_H */
